﻿using ProjectMonorail.Scripts.Exceptions;
using ProjectMonorail.Scripts.Monorail.Drawnings;

namespace ProjectMonorail.Scripts.Monorail.CollectionGenericObjects
{
    /// <summary>
    /// Абстракция компании, хранящий коллекцию монорельсов
    /// </summary>
    public abstract class AbstractCompany
    {
        /// <summary>
        /// Размер места (ширина)
        /// </summary>
        protected readonly int _placeSizeWidth = 240;

        /// <summary>
        /// Размер места (высота)
        /// </summary>
        protected readonly int _placeSizeHeight = 80;

        /// <summary>
        /// Ширина окна
        /// </summary>
        protected readonly int _pictureWidth;

        /// <summary>
        /// Высота окна
        /// </summary>
        protected readonly int _pictureHeight;

        /// <summary>
        /// Коллекция монорельсов
        /// </summary>
        protected ICollectionGenericObjects<DrawingMonorail>? _collection = null;

        /// <summary>
        /// Вычисление максимального количества элементов, который можно разместить в окне
        /// </summary>
        //private int GetMaxCount => _pictureWidth * _pictureHeight / (_placeSizeWidth * _placeSizeHeight);
        private int GetMaxCount => 40;

        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="pictureWidth">Ширина окна</param>
        /// <param name="pictureHeight">Высота окна</param>
        /// <param name="collection">Коллекция монорельсов</param>
        public AbstractCompany(int pictureWidth, int pictureHeight, ICollectionGenericObjects<DrawingMonorail> collection)
        {
            _pictureWidth = pictureWidth;
            _pictureHeight = pictureHeight;
            _collection = collection;
            _collection.MaxCount = GetMaxCount;
        }

        /// <summary>
        /// Перегрузка оператора сложения для класса
        /// </summary>
        /// <param name="company">Компания</param>
        /// <param name="monorail">Добавляемый объект</param>
        /// <returns></returns>
        public static int operator +(AbstractCompany company, DrawingMonorail monorail)
        {
            return company._collection?.Insert(monorail, new DrawiningMonorailEqutables()) ?? -1;
        }

        /// <summary>
        /// Перегрузка оператора удаления для класса
        /// </summary>
        /// <param name="company">Компания</param>
        /// <param name="position">Номер удаляемого объекта</param>
        /// <returns></returns>
        public static DrawingMonorail operator -(AbstractCompany company, int position)
        {
            return company._collection?.Remove(position) ?? throw new ObjectNotFoundException();
        }

        /// <summary>
        /// Получение случайного объекта из коллекции
        /// </summary>
        /// <returns></returns>
        public DrawingMonorail? GetRandomObject()
        {
            Random rnd = new();
            return _collection?.Get(rnd.Next(GetMaxCount));
        }

        /// <summary>
        /// Вывод всей коллекции
        /// </summary>
        /// <returns></returns>
        public Bitmap? Show()
        {
            Bitmap bitmap = new(_pictureWidth, _pictureHeight);
            Graphics graphics = Graphics.FromImage(bitmap);
            DrawBackgound(graphics);
            SetObjectsPosition();
            for (int i = 0; i < (_collection?.Count ?? 0); ++i)
            {
                DrawingMonorail? obj = _collection?.Get(i);
                obj?.DrawTransport(graphics);
            }
            return bitmap;
        }

        /// <summary>
        /// Вывод заднего фона
        /// </summary>
        /// <param name="g"></param>
        protected abstract void DrawBackgound(Graphics g);

        /// <summary>
        /// Расстановка объектов
        /// </summary>
        protected abstract void SetObjectsPosition();

        /// <summary>
        /// Сортировка
        /// </summary>
        /// <param name="comparer">Сравнитель объектов</param>
        public void Sort(IComparer<DrawingMonorail?> comparer) => _collection?.CollectionSort(comparer);
    }
}
