﻿using ProjectMonorail.Scripts.Exceptions;
using ProjectMonorail.Scripts.Monorail.Drawnings;
using System.Diagnostics;

namespace ProjectMonorail.Scripts.Monorail.CollectionGenericObjects
{
    public class MassiveGenericObjects<T> : ICollectionGenericObjects<T>
        where T : class
    {
        /// <summary>
        /// Массив объектов, которые храним
        /// </summary>
        private T?[] _collection;

        public int Count => _collection.Length;

        public int MaxCount
        {
            get
            {
                return _collection.Length;
            }
            set
            {
                if (value > 0)
                {
                    if (_collection.Length > 0)
                    {
                        Array.Resize(ref _collection, value);
                    }
                    else
                    {
                        _collection = new T?[value];
                    }
                }
            }
        }

        public CollectionType GetCollectionType => CollectionType.Massive;


        /// <summary>
        /// Конструктор
        /// </summary>
        public MassiveGenericObjects()
        {
            _collection = Array.Empty<T>();
        }

        public T? Get(int position)
        {
            // TODO проверка позиции
            if (!(position >= 0 && position < Count) || _collection[position] == null) return null; 
            return _collection[position];
        }

        public int Insert(T obj, IEqualityComparer<T?>? comparer = null)
        {
            // TODO вставка в свободное место набора
            if (comparer != null)
            {
                foreach (T? item in _collection)
                {
                    if ((comparer as IEqualityComparer<DrawingMonorail>).Equals(obj as DrawingMonorail, item as DrawingMonorail))
                    {
                        throw new ObjectIsEqualException();
                    }
                }
            }

            for (int i = 0; i < Count; i++)
            {
                if (InsertingElementCollection(i, obj)) return i; 
            }

            throw new CollectionOverflowException("Превышение лимита Count");
        }

        public int Insert(T obj, int position, IEqualityComparer<T?>? comparer = null)
        {
            // TODO проверка позиции
            // TODO проверка, что элемент массива по этой позиции пустой, если нет, то
            // ищется свободное место после этой позиции и идет вставка туда
            // если нет после, ищем до
            // TODO вставка
            if (comparer != null)
            {
                foreach (T? item in _collection)
                {
                    if ((comparer as IEqualityComparer<DrawingMonorail>).Equals(obj as DrawingMonorail, item as DrawingMonorail))
                        throw new ObjectIsEqualException();
                }
            }

            if (!(position >= 0 && position < Count)) throw new PositionOutOfCollectionException(position);
            if (InsertingElementCollection(position, obj)) return position;

            for (int i = position + 1; i < Count; i++)
            {
                if (InsertingElementCollection(i, obj)) return i;
            }

            for (int i = position - 1; i >= 0; i--)
            {
                if (InsertingElementCollection(i, obj)) return i;
            }

            throw new CollectionOverflowException("Нет свободного места для вставки");
        }

        public T Remove(int position)
        {
            // TODO проверка позиции
            // TODO удаление объекта из массива, присвоив элементу массива значение null

            if (!(position >= 0 && position < Count) || _collection[position] == null) throw new ObjectNotFoundException(position);

            T obj = _collection[position];
            _collection[position] = null;
            
            return obj;
        }

        /// <summary>
        /// Если элемент массива пустой, то происходит вставка нового элемента
        /// </summary>
        /// <param name="index">Индекс элемента</param>
        /// <param name="obj">Элемент</param>
        /// <returns>false - элемент массива не равен null, true - равен null</returns>
        private bool InsertingElementCollection(int index, T obj)
        {
            if (_collection[index] != null) return false;

            _collection[index] = obj;
            return true;
        }

        public IEnumerable<T?> GetItems()
        {
            for (int i = 0; i < _collection.Length; ++i)
            {
                yield return _collection[i];
            }
        }

        void ICollectionGenericObjects<T>.CollectionSort(IComparer<T?> comparer)
        {
            Array.Sort(_collection, comparer);
        }
    }
}