﻿namespace ProjectMonorail.Scripts.Monorail.Entities
{
    /// <summary>
    /// Класс-сущности "Современный монорельс" 
    /// </summary>
    public class EntityModernMonorail : EntityMonorail
    {
        /// <summary>
        /// Дополнительный цвет (для опциональных элементов)
        /// </summary>
        public Color AdditionalColor { get; private set; }

        /// <summary>
        /// Признак (опция) наличия монорельсового пути
        /// </summary>
        public bool MonorailTrack { get; private set; }

        /// <summary>
        /// Признак (опция) наличия кабины
        /// </summary>
        public bool Cabin { get; private set; }

        public void SetAdditionalColor(Color value)
        {
            AdditionalColor = value;
        }

        /// <summary>
        /// Инициальзация полей объекта-класса спортивного автомобиля
        /// </summary>
        /// <param name="speed">Скорость</param>
        /// <param name="weight">Вес автомобиля</param>
        /// <param name="bodyColor">Основной цвет</param>
        /// <param name="additionalColor">Дополнительный цвет</param>
        /// <param name="monorailTrack">Признак наличия монорельсового пути</param>
        /// <param name="cabin">Признак наличия кабины</param>
        public EntityModernMonorail(int speed, double weight, Color bodyColor, Color additionalColor, bool monorailTrack, bool cabin) : base(speed, weight, bodyColor)
        {
            AdditionalColor = additionalColor;
            MonorailTrack = monorailTrack;
            Cabin = cabin;
        }

        /// <summary>
        /// Получение строк с значениями свойств объекта класса-сущности
        /// </summary>
        /// <returns></returns>
        public override string[] GetStringRepresention()
        {
            return new string[] { nameof(EntityModernMonorail), Speed.ToString(), Weight.ToString(), BodyColor.Name.ToString(), AdditionalColor.Name.ToString(), MonorailTrack.ToString(), Cabin.ToString()};
        }

        /// <summary>
        /// Создание объекта из массива строк
        /// </summary>
        /// <param name="args"></param>
        /// <returns></returns>
        public static EntityModernMonorail? CreateEntityModernMonorail(string[] args)
        {
            if (args.Length == 0 || args[0] != nameof(EntityModernMonorail)) return null;

            return new EntityModernMonorail(Convert.ToInt32(args[1]), Convert.ToDouble(args[2]), Color.FromName(args[3]), Color.FromName(args[4]), Convert.ToBoolean(args[5]), Convert.ToBoolean(args[6]));
        }
    }
}
