﻿namespace ProjectMonorail.Scripts.Monorail.Entities
{
    /// <summary>
    /// Класс-сущности "Монорельс"
    /// </summary>
    public class EntityMonorail
    {
        /// <summary>
        /// Скорость
        /// </summary>
        public int Speed { get; private set; }

        /// <summary>
        /// Вес
        /// </summary>
        public double Weight { get; private set; }

        /// <summary>
        /// Основной цвет
        /// </summary>
        public Color BodyColor { get; private set; }

        /// <summary>
        /// Шаг перемещения автомобиля
        /// </summary>
        public double Step => Speed * 100 / Weight;

        public void SetBodyColor(Color value)
        {
            BodyColor = value;
        }

        /// <summary>
        /// Конструктор сущности
        /// </summary>
        /// <param name="speed">Скорость</param>
        /// <param name="weight">Вес автомобиля</param>
        /// <param name="bodyColor">Основной цвет</param>
        public EntityMonorail(int speed, double weight, Color bodyColor)
        {
            Speed = speed;
            Weight = weight;
            BodyColor = bodyColor;
        }

        /// <summary>
        /// Получение строк с значениями свойств объекта класса-сущности
        /// </summary>
        /// <returns></returns>
        public virtual string[] GetStringRepresention()
        {
            return new string[] { nameof(EntityMonorail), Speed.ToString(), Weight.ToString(), BodyColor.Name.ToString() };
        }

        /// <summary>
        /// Создание объекта из массива строк
        /// </summary>
        /// <param name="args"></param>
        /// <returns></returns>
        public static EntityMonorail? CreateEntityMonorail(string[] args)
        {
            if (args.Length == 0 || args[0] != nameof(EntityMonorail)) return null;

            return new EntityMonorail(Convert.ToInt32(args[1]), Convert.ToDouble(args[2]), Color.FromName(args[3]));
        }
    }
}
