﻿using ProjectMonorail.Scripts.Monorail.Drawnings;

namespace ProjectMonorail.Scripts.Monorail.MovementStrategy
{
    /// <summary>
    /// Класс-реализация IMoveableObject с использованием DrawingMonorail
    /// </summary>
    public class MoveableMonorail : IMoveableObject
    {
        /// <summary>
        /// Поле-объект класса DrawingMonorail или его наследника
        /// </summary>
        private readonly DrawingMonorail _drawingMonorail;

        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="drawingMonorail">Объект класса DrawingMonorail</param>
        public MoveableMonorail(DrawingMonorail drawingMonorail)
        {
            _drawingMonorail = drawingMonorail;
        }

        public ObjectParameters? GetObjectPosition
        {
            get
            {
                if (_drawingMonorail == null || _drawingMonorail.EntityMonorail == null || !_drawingMonorail.GetPositionX.HasValue || !_drawingMonorail.GetPositionY.HasValue) return null;
                return new ObjectParameters(_drawingMonorail.GetPositionX.Value, _drawingMonorail.GetPositionY.Value, _drawingMonorail.GetWidth, _drawingMonorail.GetHeight);
            }
        }

        public int GetStep => (int)(_drawingMonorail?.EntityMonorail?.Step ?? 0);

        public bool TryMoveObject(MovementDirection direction)
        {
            if (_drawingMonorail == null || _drawingMonorail.EntityMonorail == null)
            {
                return false;
            }
            return _drawingMonorail.MoveTransport(GetDirectionType(direction));
        }

        /// <summary>
        /// Конвертация из MovementDirection в DirectionType
        /// </summary>
        /// <param name="direction">MovementDirection</param>
        /// <returns>DirectionType</returns>
        private static DirectionType GetDirectionType(MovementDirection direction)
        {
            return direction switch
            {
                MovementDirection.Left => DirectionType.Left,
                MovementDirection.Right => DirectionType.Right,
                MovementDirection.Up => DirectionType.Up,
                MovementDirection.Down => DirectionType.Down,
                _ => DirectionType.Unknow,
            };
        }
    }
}
